SystemOrganization addCategory: #'Installer-Core'!

Object subclass: #Installer
	instanceVariableNames: 'sm wsm mc ma url user command markers password project package bug desc answers packages messagesToSuppress pageDataStream date'
	classVariableNames: 'WebSearchPath Entities IsSetToTrapErrors SkipLoadingTests Fixes Bindings'
	poolDictionaries: ''
	category: 'Installer-Core'!

!Installer commentStamp: 'kph 1/7/2007 22:53' prior: 0!
Installer provides a simple Domain Specific Language for installing packages from monticello and squeakmap of various version.

One design decision is to be able to paste scripts a workspace and run and tested from there without any special editing.

Installer can also be used as a squeak startup document with various commandline parameters.

example 1 (non working example)

unix$ squeak Squeak3.8-7067 http://installer.pbwiki.com/f/Installer.st Url="http://..." saveas="MyImage" postinstall=Tester done=quit

This will load installer as the startup document, installer will install a file/script from the given url, then the image will be saved in the given filename, and processing is then passed on to the class Tester, once complete the image will quit.

supported parameters:
P|PATH=prefix1*suffix1;...;prefixN*suffixN
I|IN|INSTALL=aPage (needs PATH)
U|URL=aUrl
O|OUT|SAVEAS=aFileName
+S|SAVE
P|TEST|POSTINSTALL=aClassName | aPage(given PATH) | aUrl
DONE= save | quit | save&quit | aClassName
+SKIPTESTS

example 2: (working example)

unix$ squeak Squeak3.8-7067 http://installer.pbwiki.com/f/Installer.st path=wiki.squeak.org/squeak/;installer.pbwiki.com/*-alt in=CommandLineExample2 done=quit.

===
Examples - Using SqueakMap

1)
squeakmap := Installer squeakmap.
squeakmap install: 'DynamicBindings'.
squeakmap open. "opens the squeak map loader gui"
2)
Installer squeakmap install: 'DynamicBindings'.

3) Alternatively using websqueakmap which uses http (similar usage api to squeakmap)
squeakmap := Installer websqueakmap.
squeakmap package: 'DynamicBindings'; install.

4) Abbreviated instanciation
squeakmap := Installer sm. "squeakmap"
squeakmap := Installer wsm. "web-squeakmap"

5) Finding Packages by Name
(Installer sm match: 'Labby*') explore.

6) Obtaining List of Package Versions
(Installer sm package: 'Labby & Walker'; versions)  explore.  

6) Searching Packages
Installer sm search: 'seaside'.
Installer sm search: 'author:*Smith'. 
"fields available for searching: author: name:, summary:,description:"

7) Specify specific version of a package for installation
Installer sm install: 'Labby(17)'.

Examples - Using Monticello
1)
squeaksource := Installer repository: 'http://www.squeaksource.com'.
seaside := squeaksource project: 'Seaside'.
seaside install: 'Comet-lr.8';
		install: 'Scriptaculous'.
2) 
squeaksource := Installer repository: 'http://www.squeaksource.com'		
squeaksource project: 'Seaside';
			  install: 'Comet-lr.8;
			  install: 'Scriptaculous'.
3) Supplying user/password.
squeaksource := Installer repository: 'http://www.squeaksource.com'.
squeaksource user: 'me'; password: 'asecret'.

4) Abbreviated instanciation for convenience.
squeaksource := Installer squeakSource.
or
squeaksource := Installer ss.

Examples - install straight from a url		
1a)
(Installer url: 'http://minnow.cc.gatech.edu/squeak/uploads/5889/MakeTestsGreen39.cs') install.
1b)
Installer installUrl: 'http://minnow.cc.gatech.edu/squeak/uploads/5889/MakeTestsGreen39.cs'.

2)
| page |
page := Installer url: 'http://minnow.cc.gatech.edu/squeak/uploads/5889/'.
page package: 'MakeTestsGreen39.cs'.
page install.

3)
In html page scripts are delimited by <code st>...</code st>
| page |
page := Installer url: 'http://wiki.squeak.org/squeak/742'.
page install.
4)
specify your own delimeters
| page |
page := Installer url: 'http://wiki.squeak.org/742'.
page markers: 'beginning of script...end of script'.
page install.

Note: Scripts embedded in html or a swiki page may need to escape some entities.
Supported entities are &amp; &gt; &lt; &star; &quot; (see Installer-c-entities)

Examples - Using Mantis
1 . Viewing a file uploaded to a mantis bug report

Installer mantis bug: 4874 view: 'Join.4.cs'.

1. Installing a file uploaded to a mantis bug report
Installer mantis bug: 4874 fix: 'Join.4.cs'.

2. as above, inform user if bug report has been updated since a given date 
Installer mantix	bug: 4874 fix: 'JoinTest.1.cs' date: '12-18-06'. 
	
3. Install or view a fix for a given bug.

Installer mantis viewBug: 4874.
Installer mantis fixBug: 4874.

The fix script is published in a note added to the bug report page with the following syntax

"fix begin"
 Installer mantis bug: 474 fix: 'Join.4.cs'.
"fix test"
 Installer mantis bug: 474 fix: 'JoinTest.1.cs' date: '12-18-06'. 
"fix end"

4. Install a fix for a given bug, ignoring all test code
(test code being delimeted by: "fix test" ... "fix end" )

Installer mantis justFixBug: 474.
or
Installer mantis justFixBug: 474 date: '12-19-06'.



!

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 12/21/2006 02:08'!
cancelSkipLoadingTests

"sets a flag to un-ignore loading of the testing portion of scripts embedded in pages"
SkipLoadingTests := false.
 ! !

!Installer class methodsFor: 'accessing system' stamp: 'kph 2/14/2007 07:36'!
classProjectLauncher

^Smalltalk at: #ProjectLauncher ifAbsent: [ self error: 'ProjectLauncher not present' ]! !

!Installer class methodsFor: 'accessing system' stamp: 'kph 1/5/2007 02:52'!
classes

^ Smalltalk! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 1/11/2007 00:17'!
do: webPageName

self webSearchPath isEmpty ifTrue: [ ^self error: 'search path not set' ].

^(self new package: webPageName) install.
! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 12/23/2006 05:46'!
entities

^ Entities ifNil: [ Entities := 
"enough entities to be going on with"
  Dictionary new.
Entities at: 'lt' put: '<';
	at: 'gt' put: '>';
	at: 'amp' put: '&';
	at: 'star' put: '*';
	at: 'quot' put: '"';
	at: 'nbsp' put: ' ';
 	yourself
]

 ! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 1/7/2007 16:13'!
fixesApplied

^ Fixes ifNil: [ Fixes := OrderedCollection new ].! !

!Installer class methodsFor: 'documentation' stamp: 'kph 1/10/2007 04:52'!
history

"
7 Jan 2007  
!!Installer fixBug: <aBugNo>

aBugNo can now be a number or a string, beginning with a number. 
This allows the mantis bug report summary to be used verbatim.
It also provides more infomarion for Installer to support self documentation.

!!Install fix if not already installed
 Installer ensureFix: <aBugNoOrString>
 Installer ensureFixes: #(1 2 3 4)

Installer now keeps a list of fix <aBugNoOrString> that have been installed up to this point.
#ensureFix: will only install the fix if it has not already been loaded.
note that only the bugNumber not the description is significant in the check.

8 Jan 2007
!!Installer view: <webPageNameOrUrl>

Provided that web page based scripts follow some simple rules, installer can collate the scripts from 
web pages into a single workspace where you can manually 'doit' portions as you wish.

The report generation is not very clever, it only matches on:
 'Installer install:' ,  'Installer installUrl:', and 'Installer mantis fixBug:'
 note these lines must be properly completed with an ending $. (period).

also invoked by commandline option VIEW=<webPageNameOrUrl>

10 Jan 2007
!!Now matches simpler html

Check for an html page, now matches
'<!!DOCTYPE HTML' and <html> 
the allows use of pbwiki's raw=bare option which returns iframe 
embeddable html without the usual headers.
"! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 2/16/2007 03:51'!
initialize

	"Establish whether we are running as part of the squeak initialization process"

	Smalltalk garbageCollect.
	self classProjectLauncher allInstances size = 0 ifTrue: [ ^ self ].

	"we have been launched at the command line - we need launcher!!"
	
	
	Smalltalk at: #Launcher ifPresent: [ :l | ^self ]. "Launcher is here already, no need to worry"

	[
	self squeaksource project: 'Installer'; install: 'Launcher-Core'.
	
	] on: Warning do: [ :ex | ex resume ].
	! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 1/8/2007 07:42'!
install: webPageName

self webSearchPath isEmpty ifTrue: [ ^self error: 'search path not set' ].

^(self new package: webPageName) install.
! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 1/8/2007 07:42'!
installUrl: urlString

^ (self url: urlString) install.
! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 10/19/2006 06:55'!
mantis

^self mantis: 'http://bugs.impara.de/'! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 12/21/2006 04:26'!
mantis: host

^self new	ma: host; 
			markers: '&quot;fix begin&quot;...&quot;fix test&quot;...&quot;fix end&quot;'; 
			yourself.
! !

!Installer class methodsFor: 'accessing' stamp: 'kph 1/16/2007 19:28'!
path: aString
	"convenience abbreviation"
	self webSearchPathFrom: aString! !

!Installer class methodsFor: 'accessing' stamp: 'kph 11/12/2006 16:37'!
repository: host  

^self new mc: host ! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 12/21/2006 01:49'!
skipLoadingTests

"sets a flag to ignore loading of the testing portion of scripts embedded in pages"
SkipLoadingTests := true.
 ! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 1/4/2007 06:42'!
skipLoadingTests: yesNo during: block

	| oldValue |

	oldValue := SkipLoadingTests.
	SkipLoadingTests := yesNo.
	
	block ensure:[ SkipLoadingTests := oldValue ].! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 11/12/2006 16:30'!
sm

^self squeakmap! !

!Installer class methodsFor: 'accessing system' stamp: 'kph 1/5/2007 02:50'!
smalltalkImage

^ SmalltalkImage current! !

!Installer class methodsFor: 'accessing system' stamp: 'kph 1/5/2007 02:53'!
sourceFiles

^ SourceFiles! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 10/15/2006 06:37'!
squeakmap

^self new sm: true; yourself! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 11/17/2006 01:52'!
squeaksource

^self repository: 'http://www.squeaksource.com'! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 11/17/2006 01:53'!
ss

^self squeaksource
 ! !

!Installer class methodsFor: 'utility' stamp: 'kph 1/10/2007 15:27'!
unload: categoryMatchesString 

^ self new unload: categoryMatchesString! !

!Installer class methodsFor: 'accessing' stamp: 'kph 11/10/2006 00:48'!
url: urlString

^self new url: urlString; yourself! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 1/8/2007 08:37'!
view: webPageNameOrUrl

| theReport |

theReport := String streamContents: [ :report | 
	(webPageNameOrUrl beginsWith: 'http://') ifTrue: [ 
		self new urlAction: ('Installer installUrl: ', (webPageNameOrUrl printString),'.')  	
						 reportOn: report.
	]
	ifFalse: [
		self new webAction: ('Installer install: ', (webPageNameOrUrl printString),'.')  	
						 reportOn: report.
	].
].

Workspace new contents: (theReport contents); openLabel: webPageNameOrUrl.

^theReport contents
! !

!Installer class methodsFor: 'accessing' stamp: 'kph 1/5/2007 00:18'!
webSearchPath

"a search path item, has the following format. prefix*suffix"

^ WebSearchPath ifNil: [ WebSearchPath := OrderedCollection new ].! !

!Installer class methodsFor: 'accessing' stamp: 'kph 1/5/2007 04:19'!
webSearchPathFrom: string

	| reader result |
	
	reader := string readStream.
	result := OrderedCollection new.
	
	[ reader atEnd ] whileFalse: [ result add: (reader upTo: $;) ].

	WebSearchPath := result.! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 10/21/2006 01:38'!
websqueakmap

^self new wsm: 'http://map.squeak.org'; yourself! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 10/21/2006 01:38'!
websqueakmap: host

^self new wsm: host; yourself! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 11/12/2006 16:35'!
wsm

^self websqueakmap! !

!Installer methodsFor: 'action report' stamp: 'kph 1/11/2007 00:22'!
actionMatch: theLine reportOn: report ifNoMatch: aBlock

	| line |
	
	line := theLine withBlanksCondensed.
	
	((line beginsWith: 'Installer install:') | (line beginsWith: 'Installer do:')) 
		ifTrue: [ ^self webAction: theLine reportOn: report ].
	
	
	((line beginsWith: 'Installer installUrl:') and: 
		[ | ext |
		 ext :=  (line readStream upToAll: '''.') copyAfterLast: $..
		 (#( 'cs' 'st' 'mcz' 'sar') includes: ext) not ]) ifTrue: [ ^self urlAction: theLine reportOn: report ].

	(line beginsWith: 'Installer mantis fixBug:') ifTrue: [ ^self mantisAction: theLine reportOn: report ].
	
	
	aBlock value.
! !

!Installer methodsFor: 'auto answering' stamp: 'kph 10/17/2006 13:07'!
answer: aString with: anAnswer

^self answers add: ( Array with: aString with: anAnswer )! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/17/2006 13:06'!
answers

^ answers ifNil: [ answers := OrderedCollection new ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 06:55'!
answers: anObject
	"Set the value of answers"

	answers := anObject! !

!Installer methodsFor: 'script bindings' stamp: 'kph 1/8/2007 00:11'!
bindingOf: aString 
	
	Bindings isNil ifTrue: [ Bindings := Dictionary new].

	(Bindings includesKey: aString)
		ifFalse: [Bindings at: aString put: nil].

	^ Bindings associationAt: aString.! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 05:06'!
bug
	"Answer the value of bug"

	^ bug! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 09:48'!
bug: aBugNo

"
Installer mantis viewBug: 5639.
"
	| page text | 

	self setBug: aBugNo.
	
	page := self maPage.
 
	text := String streamContents: [ :str |	
			
		#('Bug ID' 'Category' 'Severity' 'Reproducibility' 'Date Submitted' 
			'Date Updated' 'Reporter' 'View Status' 'Handler' 
			'Priority' 'Resolution' 'Status' 'Product Version' 'Summary' 'Description' 'Additional Information' ) 
				do: [ :field |
						| f |
						f := self maRead: page field: field.
			str nextPutAll: f key; nextPutAll: ': '; nextPutAll: f value; cr.
		].
	
	str nextPutAll: 'Notes: '; cr.
		(self maReadNotes: page) do: [ :note | str nextPutAll: note; cr; cr ].
		
		str nextPutAll: 'Files: '; nextPutAll: self maFiles keys asArray printString.
	].
 	
^ text	
! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 04:11'!
bug: aBugNo fix: aFileName

	^ self bug: aBugNo fix: aFileName date: nil! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 10:08'!
bug: aBugNo fix: aFileName date: aDate
 
	| stream |

	self setBug: aBugNo.

	self logCR: 'obtaining ', aFileName, '...'.

	stream := self maStreamForFile: aFileName.

	self ditchOldChangeSetFor: aFileName.

	self maCheckDateAgainst: aDate.

	self install: aFileName from: stream.

	self class fixesApplied add: aBugNo.
		
	^ date! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 09:49'!
bug: aBugNo retrieve: aFileName

	 self setBug: aBugNo.

	^ (self maStreamForFile: aFileName) contents! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 09:48'!
bug: aBugNo view: aFileName

	| file list |
	
	self setBug: aBugNo.
	
	file :=  self maStreamForFile: aFileName.

	list := self classChangeList new
			scanFile:  file from: 1 to: file size.
		 
	self classChangeList open: list name: (aFileName, ' mantis: ', aBugNo printString) 
		multiSelect: true.! !

!Installer methodsFor: 'class references' stamp: 'kph 12/23/2006 02:05'!
classChangeList

^Smalltalk at: #ChangeList  ifAbsent: [ self error: 'ChangeList not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 22:17'!
classChangeSet

^Smalltalk at: #ChangeSet  ifAbsent: [ self error: 'ChangeSet not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kb 12/18/2006 12:40'!
classChangeSorter

^Smalltalk at: #ChangeSorter ifAbsent: [ self error: 'ChangeSorter not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 22:16'!
classGZipReadStream

^Smalltalk at: #GZipReadStream  ifAbsent: [ self error: 'Compression not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 15:42'!
classHTTPSocket

^Smalltalk at: #HTTPSocket ifAbsent: [ self error: 'Network package not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/17/2006 03:15'!
classMCHttpRepository

^Smalltalk at: #MCHttpRepository ifAbsent: [ self error: 'Monticello not present' ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 11/17/2006 02:46'!
classMCMczReader

^Smalltalk at: #MCMczReader ifAbsent: [ nil ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 11/17/2006 02:44'!
classMczInstaller

^Smalltalk at: #MczInstaller ifAbsent: [ nil ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 11/23/2006 03:18'!
classMultiByteBinaryOrTextStream
^Smalltalk at: #MultiByteBinaryOrTextStream  ifAbsent: [ self error: 'MultiByteBinaryOrTextStream not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 15:40'!
classSARInstaller

^Smalltalk at: #SARInstaller  ifAbsent: [ self error: 'SARInstaller not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 2/1/2007 20:02'!
classSMLoader

^Smalltalk at: #SMLoader  ifAbsent: [ self error: 'SqueakMap Loader not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 15:40'!
classSMSqueakMap

^Smalltalk at: #SMSqueakMap  ifAbsent: [ self error: 'SqueakMap not present' ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 12/21/2006 02:42'!
date: anObject
	"Set the value of markers"

	date := anObject asDate! !

!Installer methodsFor: 'accessing' stamp: 'kph 1/8/2007 07:22'!
desc
	 
	^ desc! !

!Installer methodsFor: 'accessing' stamp: 'kph 1/8/2007 07:22'!
desc: anObject

	desc := anObject! !

!Installer methodsFor: 'utils' stamp: 'kph 1/30/2007 15:15'!
ditchOldChangeSetFor: aFileName 

	| changeSetName |
 
	changeSetName := (self validChangeSetName: aFileName) sansPeriodSuffix.


	(self classChangeSet named: changeSetName)
		ifNotNil: [  	
				(self logCR:'Removing old change set ', changeSetName) cr.
				self classChangeSet removeChangeSet: (self classChangeSet named: changeSetName) ].! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 10:10'!
ensureFix: aBugNo

	| fixesAppliedNumbers |
	
	self setBug: aBugNo.
	
	fixesAppliedNumbers := self class fixesApplied collect: [ :fixDesc | fixDesc asInteger ].
	
	(fixesAppliedNumbers includes: bug) ifFalse: [ self fixBug: aBugNo ]! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/7/2007 16:18'!
ensureFixes: aBugNos

	aBugNos do: [ :bugNo | self ensureFixes: bugNo ].! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 06:40'!
evaluate: stream

stream fileIn.! !

!Installer methodsFor: 'action report' stamp: 'kph 1/8/2007 11:14'!
extractFromHtml: html option: allOrLast

	|  start stop test in |

	start := self markersBegin.
	stop :=  self markersEnd.
	test := self markersTest.
			 
	in := ReadWriteStream with: String new.
		
	[ html upToAll: start; atEnd ] 
		whileFalse: [
			| chunk |
			(allOrLast == #last) ifTrue: [ in resetToStart ]. 
			chunk := html upToAll: stop.
			self isSkipLoadingTestsSet ifTrue: [ chunk := chunk readStream upToAll: test ].
			in nextPutAll: chunk. 
		 ].

	^self removeHtmlMarkupFrom: in reset
	 
! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 04:12'!
fixBug: aBugNo

	^ self fixBug: aBugNo date: nil! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 11:45'!
fixBug: aBugNo date: aDate

	self setBug: aBugNo.
	
 	self install: self maUrl from: self maScript.
	
	self maCheckDateAgainst: aDate.
	
	
	
! !

!Installer methodsFor: 'utils' stamp: 'kph 12/18/2006 06:50'!
httpGet: aUrl

	| page |

	page := self classHTTPSocket httpGet: aUrl accept: 'application/octet-stream'.  
 
	(page respondsTo: #reset)  ifFalse: [ self error: 'unable to contact web site' ].
 
	^ page
	! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/12/2006 16:16'!
info

self sm ifTrue: [ ^ self smInfo  ].
self wsm ifNotNil: [ ^ self wsmInfo  ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 1/5/2007 00:08'!
install

self logErrorDuring: [
	self mc ifNotNil: [ ^self mcInstall ].
	self wsm ifNotNil: [ ^self wsmInstall ].
	self sm ifTrue: [ ^self smInstall ].
	self url ifNotNil: [ ^self urlInstall ].
	self package ifNotNil: [ ^self webInstall ].
] ! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/18/2006 11:29'!
install: packageNameCollectionOrDetectBlock

self package: packageNameCollectionOrDetectBlock.

self install! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 06:43'!
install: aFileName from: stream

| ext installSelector |
	 
	self log: 'installing...'.
 
	ext := aFileName copyAfterLast: $..
	installSelector := ('install', ext asUppercase, ':from:') asSymbol.
	
	self withAnswersDo:	[
		(self respondsTo: installSelector)
			ifTrue: [ self perform: installSelector with: aFileName with: stream ]
			ifFalse: [ self installDefault: aFileName from: stream ].
	]. 

	self log: '.done'
 ! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/30/2007 01:06'!
installCS: aFileName from: stream

	self ditchOldChangeSetFor: aFileName.
	self newChangeSetFromStream: stream named: (self validChangeSetName: aFileName).
! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 06:42'!
installDefault: aFileName from: stream

stream fileIn.! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 06:42'!
installGZ: aFileName from: stream 

	"FileIn the contents of a gzipped stream"

	| zipped unzipped |
	 
	zipped := self classGZipReadStream on: stream.
	unzipped := MultiByteBinaryOrTextStream with: zipped contents asString.
	unzipped reset.

	self  newChangeSetFromStream: unzipped named:aFileName.

	! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 06:42'!
installMCZ: aFileName from: stream 

	self classMczInstaller ifNotNilDo: [ :reader | ^reader installStream: stream].  
	self classMCMczReader ifNotNilDo: [ :reader | ^(reader versionFromStream: stream) load]. 

	self error: 'no monticello readers available'. 
 ! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 06:42'!
installSAR: aFileName from: stream 

 | newCS |

	self classSARInstaller withCurrentChangeSetNamed: aFileName
		do: [:cs | newCS := cs. self classSARInstaller new fileInFrom: stream].
	newCS isEmpty ifTrue: [ self classChangeSet removeChangeSet: newCS ]! !

!Installer methodsFor: 'url' stamp: 'kph 1/10/2007 04:50'!
isHtmlStream: page

	"matches  '<!!DOCTYPE HTML', and <html>' "
	
	| first |	
	
	first := (page next: 14) asUppercase.
	
	^ (first = '<!!DOCTYPE HTML') | (first beginsWith: '<HTML>')
	
! !

!Installer methodsFor: 'accessing' stamp: 'kph 12/21/2006 02:07'!
isSkipLoadingTestsSet

^SkipLoadingTests ifNil: [ false ]! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/4/2007 06:43'!
justFixBug: aBugNo

	^self class skipLoadingTests: true during: [ self fixBug: aBugNo date: nil ]! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/4/2007 06:43'!
justFixBug: aBugNo date: d

	^self class skipLoadingTests: true during: [ self fixBug: aBugNo date: d ]! !

!Installer methodsFor: 'logging' stamp: 'kph 12/18/2006 04:51'!
log: text

^Transcript show: text.! !

!Installer methodsFor: 'logging' stamp: 'kph 12/18/2006 04:53'!
logCR: text

^ Transcript cr; show: text! !

!Installer methodsFor: 'logging' stamp: 'kph 1/30/2007 01:28'!
logErrorDuring: block

(IsSetToTrapErrors == false) ifTrue: [ ^ block value ].

block on: Error do: [ :e | self logCR: e messageText. ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 06:55'!
ma
	"Answer the value of ma"

	^ ma! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 07:07'!
ma: aUrl
 
	"Set the value of host"

	ma := aUrl last = $/ ifTrue: [ aUrl ] ifFalse: [ aUrl, '/' ]! !

!Installer methodsFor: 'utils' stamp: 'kph 12/21/2006 02:51'!
maCheckDateAgainst: okDate

	(okDate notNil and: [date < okDate asDate ]) ifTrue: [ self notify: 'bug ', self bug asString, ' updated on ', date printString ].
 ! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 11:37'!
maFiles
 
 | file files bugPage id  | 
 
 files := Dictionary new.
 
 bugPage := self maPage.

 [
  id := bugPage upToAll: 'href="file_download.php?file_id='; upTo: $&. 
 file := bugPage upToAll: 'amp;type=bug"' ; upTo: $<.
 
 ((file size > 1) and: [file first = $>]) ifTrue: [ files at: file copyWithoutFirst put: id ].
 
 id notEmpty ] whileTrue.

^files ! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 09:54'!
maPage
  "
  self mantis bug: 5251.
 "
 | page |

page :=  self httpGet: self maUrl.
 
date := ((self maRead: page field: 'Date Updated') value upTo: $ ) asDate.
 
^page reset! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/23/2006 04:50'!
maRead: page field: fieldKey

	 | value |
 
	value := page upToAll: ('!!-- ', fieldKey, ' -->'); upToAll: '<td'; upTo: $>; upToAll: '</td>'.
	
	page upTo: $<.
	
	page peek = $t ifTrue: [ value := page upToAll: 'td'; upTo: $>; upToAll: '</td>' ].
	  
	^Association key: fieldKey value: value withBlanksTrimmed! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/23/2006 05:54'!
maReadNotes: page 

	 |  notes note  |
 
	notes := OrderedCollection new.

	[ page upToAll: 'tr class="bugnote"'; upTo: $>.
	  page atEnd ]
		
	whileFalse: [ 
		note := (self removeHtmlMarkupFrom: (page upToAll: '</tr>') readStream) contents.
		note := note withBlanksCondensed.
		"note replaceAll: Character cr with: $ ."
		note replaceAll: Character lf with: Character cr.
		notes add: note  
	].
	
	^notes! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 10:01'!
maScript 

	^self extractFromHtml: self maPage option: #last
! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 09:48'!
maStreamForFile: aFileName

	| fileId  |

 	fileId :=  self maFiles at: aFileName ifAbsent: [ self error: aFileName, ' not found' ].

 	^ self httpGet: (self ma, 'file_download.php?file_id=' , fileId , '&type=bug').
	 ! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 09:54'!
maUrl
 
	^ url := self ma, 'view.php?id=', bug asString
 ! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 04:08'!
maUrlFor: maBugNo
 
	^ url := self ma, 'view.php?id=', maBugNo asString 
 ! !

!Installer methodsFor: 'action report' stamp: 'kph 1/8/2007 11:19'!
mantisAction: line reportOn: report
	
	| param mantis |
	
	mantis := Installer mantis.
	
	param :=  line readStream upTo: $: ; upTo: $..
	
	mantis setBug: ((param readStream upTo: $'; atEnd)
		 ifTrue: [  param ]
		 ifFalse: [ param readStream upTo: $'; upTo: $' ]).
	
	self reportSection: line on: report.
		
	report nextPutAll: (mantis replaceEntitiesIn: mantis markersBegin readStream).
	
	self reportFor: line page: mantis maScript on: report.
	
	report nextPutAll: (mantis replaceEntitiesIn: mantis markersEnd readStream); cr.
	! !

!Installer methodsFor: 'accessing' stamp: 'kph 1/4/2007 07:06'!
markers
	"Answer the value of markers"

	^ markers ifNil: [ '<code st>..."test ...</code st>' ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 05:06'!
markers: anObject
	"Set the value of markers"

	markers := anObject! !

!Installer methodsFor: 'url' stamp: 'kph 12/21/2006 01:54'!
markersBegin
		 	 
	 ^ self markers upTo: $.! !

!Installer methodsFor: 'url' stamp: 'kph 12/23/2006 04:02'!
markersEnd
	
	"return the third marker or the second if there are only two"
	
	| str  a |
		 	 
	 str := self markers readStream.
	 a := str upToAll: '...'; upToAll: '...'.
	 str atEnd  ifTrue: [ ^a ] ifFalse: [ ^str upToEnd ]
	! !

!Installer methodsFor: 'url' stamp: 'kph 12/21/2006 01:56'!
markersTest
		 	 
	 ^ self markers readStream upToAll: '...'; upToAll: '...'! !

!Installer methodsFor: 'searching' stamp: 'kph 1/4/2007 23:59'!
match: aMatch

	^self packagesMatching: aMatch! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:38'!
mc
	"Answer the value of mc"

	^ mc! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 07:07'!
mc: aUrl
 
	"Set the value of host"

	mc := aUrl last = $/ ifTrue: [ aUrl ] ifFalse: [ aUrl, '/' ]! !

!Installer methodsFor: 'monticello' stamp: 'kph 11/17/2006 03:16'!
mcInstall

	| repository sortMczs files fileToLoad  version detectFileBlock  count |

	self logCR: 'installing ', self package asString, '...'.

	self package isString ifTrue: [ detectFileBlock := [ :file | file beginsWith: self package ] ].
	(self package isKindOf: Array) 
			ifTrue: [ detectFileBlock :=  [ :file | (self package detect: [ :item | file beginsWith: item ] ifNone: [ false ]) ~= false ] ].
	self package isBlock ifTrue: [ detectFileBlock := self package ].
  

	repository := self mcRepository.

	sortMczs := [:a :b | 
        	[(a findBetweenSubStrs: #($.)) allButLast last asInteger > (b findBetweenSubStrs: #($.)) allButLast last asInteger] on: Error do: [:ex | false]].

	"several attempts to read files - repository readableFileNames sometimes fails"

	count := 0. fileToLoad := nil.
	
	[count := count + 1.
	 (fileToLoad = nil) and:[ count < 5 ] ] 
		whileTrue: [
						files := repository readableFileNames asSortedCollection: sortMczs.
						fileToLoad := files detect: detectFileBlock ifNone: [ nil ].
																						].

	version := repository versionFromFileNamed: fileToLoad.
	version workingCopy repositoryGroup addRepository: repository.
	repository creationTemplate: 'MCHttpRepository
        	location: ''', self mcUrl, '''
        	user: ''', self user, '''
        	password: ''', self password, ''''.
	self log: ' ', version fileName, '...'.

	self withAnswersDo: [ version load ].

	self log: 'done'.
! !

!Installer methodsFor: 'monticello' stamp: 'kph 11/16/2006 17:47'!
mcRepository

^self classMCHttpRepository location: (self mcUrl) user: self user password: self password.! !

!Installer methodsFor: 'monticello' stamp: 'kph 12/17/2006 17:58'!
mcUrl

^ self mc, (self project ifNil: [''])! !

!Installer methodsFor: 'monticello' stamp: 'kph 10/21/2006 05:47'!
mcVersions

^ (self packages select: [ :p | ( self package,'-*.mcz' ) match: p ]) collect: [ :p | self copy package: p  ; yourself ].
 ! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/17/2006 14:30'!
messagesToSuppress

^ messagesToSuppress ifNil: [ messagesToSuppress := OrderedCollection new ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 06:55'!
messagesToSuppress: anObject
	"Set the value of messagesToSuppress"

	messagesToSuppress := anObject! !

!Installer methodsFor: 'mantis' stamp: 'kb 12/18/2006 13:01'!
newChangeSetFromStream: aStream named: aName 

	"This code is based upon ChangeSet-c-#newChangesFromStream:named: which is in 3.9,
	implemented here for previous versions. The second branch is for 3.8, where ChangeSets
	are loaded by ChangeSorter. "

	| oldChanges newName newSet newStream |

	(self classChangeSet respondsTo: #newChangesFromStream:named:) 
		ifTrue: [ ^self classChangeSet newChangesFromStream: aStream named:aName ].

	(self classChangeSorter respondsTo: #newChangesFromStream:named:)
		ifTrue: [ ^self classChangeSorter newChangesFromStream: aStream named: aName ].

	oldChanges := ChangeSet current.
 
	"so a Bumper update can find it"
	newName := aName sansPeriodSuffix.

	newSet := self classChangeSet basicNewNamed: newName.

	[newSet
		ifNotNil: [(aStream respondsTo: #converter:)
				ifTrue: [newStream := aStream]
				ifFalse: [newStream := self classMultiByteBinaryOrTextStream with: aStream contentsOfEntireFile.
					newStream reset].
			self classChangeSet newChanges: newSet.
			newStream setConverterForCode.
			newStream fileInAnnouncing: 'Loading ' , newName , '...'.
			Transcript cr; show: 'File ' , aName , ' successfully filed in to change set ' , newName].
	aStream close]
		ensure: [self classChangeSet newChanges: oldChanges].
	 
	^ newSet! !

!Installer methodsFor: 'accessing' stamp: 'kph 2/1/2007 20:01'!
open
	
self sm ifTrue: [ self classSMLoader open ].
self mc ifNotNil: [self mcRepository morphicOpen: nil ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 04:05'!
package
	"Answer the value of package"

	^ package! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 04:05'!
package: anObject
	"Set the value of package"

	package := anObject! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/12/2006 16:23'!
packages
	
self sm ifTrue: [ ^self smPackages ].
self mc ifNotNil: [ ^self mcRepository allFileNames ].
self wsm ifNotNil: [ ^self wsmPackagesByName keys ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 05:06'!
packages: anObject
	"Set the value of packages"

	packages := anObject! !

!Installer methodsFor: 'searching' stamp: 'kph 11/12/2006 23:19'!
packagesMatching: aMatch

self sm ifTrue: [ ^ (self packages select: [ :p | aMatch match: p name ]) collect: [ :p | self copy package: p name; yourself ] ].
self mc ifNotNil: [ ^ (self packages select: [ :p | ( aMatch , '.mcz' ) match: p ]) collect: [ :p | self copy package: p ; yourself ] ].
self wsm ifNotNil: [ ^ (self packages select: [ :p | ( aMatch) match: p ]) collect: [ :p | self copy package: p ; yourself ] ].

^'search type not supported'! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 04:16'!
password
	"Answer the value of password"

	^ password  ifNil: [ '' ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 03:56'!
password: anObject
	"Set the value of password"

	password := anObject! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/8/2006 00:56'!
printOn: s
 
s nextPutAll: '(Installer'.

self sm ifTrue: [ s nextPutAll: ' squeakmap' ].
self ma ifNotNil: [ s nextPutAll: ' mantis' ].
self wsm ifNotNil: [ s nextPutAll: ' websqueakmap' ].
self url ifNotNil: [ s nextPutAll: ' url:''', self url,'''' ].
self mc ifNotNil: [ s nextPutAll: ' repository:''', self mc,'''' ].

s nextPut: $).

self project ifNotNil: [ s nextPutAll: ' project:';
  						nextPutAll: '''', self project, ''''.
					   self package ifNotNil: [ s nextPutAll: '; '] ].
					
self package ifNotNil: [ s nextPutAll: ' package:';
  						 nextPutAll: '''', self package asString, '''' ].
					
s nextPut: $..! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:30'!
project
	"Answer the value of project"

	^ project! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:33'!
project: anObject
	"Set the value of project"

	project := anObject.

	^self copy.! !

!Installer methodsFor: 'url' stamp: 'kph 1/8/2007 11:16'!
removeHtmlMarkupFrom: in 

	| out |
 
	out := ReadWriteStream on: (String new: 100).
	[ in atEnd ] whileFalse: [ 
		out nextPutAll: (in upTo: $<).
		(((in upTo: $>) asLowercase beginsWith: 'br') and: [ (in peek = Character cr) ]) ifTrue: [ in next ].	
	].
	
	^self replaceEntitiesIn: out reset.
! !

!Installer methodsFor: 'url' stamp: 'kph 1/8/2007 11:15'!
replaceEntitiesIn: in

	| out |
	
	out := ReadWriteStream on: (String new: 100).
	[ in atEnd ] whileFalse: [ 
		out nextPutAll: ((in upTo: $&) replaceAll: Character lf with: Character cr).
		in atEnd ifFalse: [ out nextPutAll: (self class entities at: (in upTo: $;) ifAbsent: '?') ].	
	].

	^out reset! !

!Installer methodsFor: 'action report' stamp: 'kph 1/8/2007 08:17'!
reportFor: theLine page: thePage on: report 
 	
	[ thePage atEnd ] whileFalse: [ 
		| line |
		line := thePage nextLine.
		self actionMatch: line reportOn: report ifNoMatch: [ report nextPutAll: line; cr. ] 	
	].! !

!Installer methodsFor: 'action report' stamp: 'kph 1/8/2007 11:20'!
reportSection: line on: report
	
	report isEmpty ifFalse: [ report cr ].
	
	report nextPutAll: '">>>> ' ; nextPutAll: (line copyWithout: $"); nextPut: $"; cr.

	! !

!Installer methodsFor: 'searching' stamp: 'kph 11/12/2006 23:19'!
search: aMatch

self sm ifTrue: [ ^ self smSearch: ('*',aMatch,'*') ].
self mc ifTrue: [ ^ self mcSearch: ('*',aMatch,'*')  ].
self wsm ifNotNil: [ ^ self wsmSearch: ('*',aMatch,'*')  ].

^'search type not supported'! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/30/2007 14:18'!
setBug: stringOrNumber
 
  | str |
 
 stringOrNumber isInteger ifTrue: [ bug := stringOrNumber. desc := ''. ^self ].
 
 bug := stringOrNumber asInteger.
 str := str printString. 
 desc := stringOrNumber copyFrom: (str size + 1) to: (stringOrNumber size) ! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/21/2006 01:28'!
skipTests

! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:55'!
sm
	"Answer the value of sm"

	^ sm ifNil: [ false ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:38'!
sm: anObject
	"Set the value of sm"

	sm := anObject! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 10/21/2006 06:54'!
smInstall 

	| pkgAndVersion releases release |

	pkgAndVersion := self smPackageAndVersion.

	self logCR: 'installing ', self package, ' from SqueakMap...'.


	releases := self smReleasesForPackage: pkgAndVersion first.
 	
	release := pkgAndVersion last isEmpty ifTrue: [ releases last ]
					ifFalse:[ releases detect: [ :rel | rel version = pkgAndVersion last ] ]. 
	
	self withAnswersDo: [ release install ].

	self log: ' done'.
! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 10/17/2006 15:24'!
smPackageAndVersion

| p |

p := ReadStream on: self package .

^Array with: (p upTo: $() with: (p upTo: $)).! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 2/1/2007 19:14'!
smPackages
	
^self classSMSqueakMap default packagesByName! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 11/16/2006 15:40'!
smReleasesForPackage: name 

^(self classSMSqueakMap default packageWithName: name) releases! !

!Installer methodsFor: 'searching' stamp: 'kph 11/12/2006 23:28'!
smSearch: aMatch  

	| results |
	
	results := Set new.

	self packages do: [ :pkg |
		({ 'name:',pkg name.
		   'summary:', pkg summary.
		   'description:', pkg description.
		   'author:', pkg author. } anySatisfy: [ :field | aMatch match: field ])
		 ifTrue: [ results add: (self copy package: pkg name) ]. 
	].

	^results

! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 11/12/2006 16:54'!
smVersions
 
^ (self smReleasesForPackage: self package) collect: [ :v | self copy package: (v package name,'(',v version,')'); yourself. ] 

 ! !

!Installer methodsFor: 'auto answering' stamp: 'kph 10/17/2006 14:30'!
suppress: aMessage

messagesToSuppress add: aMessage! !

!Installer methodsFor: 'script bindings' stamp: 'kph 1/11/2007 02:03'!
unload: categoryMatchesString 

 (SystemOrganization categoriesMatching: categoryMatchesString) do: [ :cat | 
     self logCR: 'Unloading ', cat.
 	(MCPackage named: cat)  workingCopy unload.
	SystemOrganization removeCategory: cat.
 ].

"Until Mantis 5718 is addressed"
 Smalltalk at: #PackagePaneBrowser ifPresent: [ :ppbClass | ppbClass allInstancesDo: [ :ppb | ppb updatePackages ]  ].
 Smalltalk at: #Browser ifPresent: [ :bClass | bClass allInstancesDo: [ :b | b updateSystemCategories ] ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 04:23'!
url

^url! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 04:23'!
url: aUrl
 
 url := aUrl! !

!Installer methodsFor: 'action report' stamp: 'kph 1/8/2007 11:19'!
urlAction: line reportOn: report 
 
	url :=  line readStream upTo: $' ; upTo: $'.
  	
	self reportSection: line on: report.

	(pageDataStream := self urlGet: self urlToDownload) 
		ifNil: [ self error: 'unable to contact host' ].
	 	
	self reportFor: line page: pageDataStream on: report ! !

!Installer methodsFor: 'url' stamp: 'kph 1/7/2007 02:15'!
urlGet

^ self urlGet: self urlToDownload! !

!Installer methodsFor: 'url' stamp: 'kph 1/10/2007 04:35'!
urlGet: aUrl

	| page |

	page := HTTPSocket httpGet: aUrl accept: 'application/octet-stream'.  
 
	(page respondsTo: #reset)  ifFalse: [ ^ nil ].
 
	(self isHtmlStream: page) ifTrue: [ page := self extractFromHtml: page option: nil ].

	^ page reset
	! !

!Installer methodsFor: 'url' stamp: 'kph 1/8/2007 07:03'!
urlInstall 
"
Installer installUrl: 'wiki.squeak.org/742'.
"
 
	self logCR: 'retrieving ', self urlToDownload , ' ...'.
	
	(pageDataStream := self urlGet: self urlToDownload) 
		ifNil: [ self error: 'unable to contact host' ].
	 
	self install: self urlToDownload from: pageDataStream.
	
	^ pageDataStream 
! !

!Installer methodsFor: 'url' stamp: 'kph 1/4/2007 23:49'!
urlToDownload

	^ (self url, (self package ifNil: [ '' ])) asUrl asString.
	
 ! !

!Installer methodsFor: 'url' stamp: 'kph 1/7/2007 02:16'!
urlView

^ self urlGet: self urlToDownload! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 04:15'!
user
	"Answer the value of user"

	^ user ifNil: [ '' ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 03:56'!
user: anObject
	"Set the value of user"

	user := anObject! !

!Installer methodsFor: 'url' stamp: 'kb 12/18/2006 13:30'!
validChangeSetName: aFileName
	" dots in the url confuses the changeset loader. I replace them with dashes"
 	self url ifNotNil: [ | asUrl |
		asUrl := Url absoluteFromText: aFileName.
		^String streamContents: [:stream |
			stream nextPutAll: (asUrl authority copyReplaceAll: '.' with: '-').
			asUrl path allButLastDo: [:each |
				stream
					nextPutAll: '/';
					nextPutAll: (each copyReplaceAll: '.' with: '-') ].
			stream
				nextPutAll: '/';
				nextPutAll: asUrl path last ] ].
	^aFileName! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/21/2006 05:50'!
versions
	
self sm ifTrue: [ ^self smVersions ].
self mc ifNotNil: [ ^self mcVersions ].
self wsm ifNotNil: [ ^self wsmVersions ].! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/23/2006 06:02'!
viewBug: aBugNo

	^Workspace new contents: (self bug: aBugNo); openLabel: ('Mantis ', aBugNo printString).
! !

!Installer methodsFor: 'utils' stamp: 'kph 1/7/2007 15:57'!
viewUrl

	^Workspace new contents: (self urlGet contents); openLabel: self urlToDownload.
! !

!Installer methodsFor: 'action report' stamp: 'kph 1/8/2007 11:19'!
webAction: line reportOn: report
	
	package :=  line readStream upTo: $' ; upTo: $'.

	self reportSection: line on: report.
	
 	url := self webFindUrlToDownload.
	
	self reportFor: line page: pageDataStream on: report ! !

!Installer methodsFor: 'web install' stamp: 'kph 1/30/2007 00:53'!
webFindUrlToDownload

	self class webSearchPath do: [ :pathSpec | 
		| potentialUrl readPathSpec  |
	
		readPathSpec := pathSpec readStream.
		potentialUrl := (readPathSpec upTo: $*), package, (readPathSpec upToEnd ifNil: [ '' ]).
	
		pageDataStream := self urlGet: potentialUrl.
		
		pageDataStream notNil ifTrue: [ ^potentialUrl ]
	].

	^nil
! !

!Installer methodsFor: 'web install' stamp: 'kph 1/30/2007 00:56'!
webInstall 
"
Installer install: 'MyPage'.
"
 	url := self webFindUrlToDownload ifNil: [ self logCR: 'url not found'. self error: 'url not found' ].
	
	self logCR: 'found ',  url, ' ...'.
	
	pageDataStream size > 0 
		ifTrue: [ self install: url from: pageDataStream ]
		ifFalse: [ self logCR: '...',url,' was empty' ].
	
	^ pageDataStream.

	! !

!Installer methodsFor: 'monticello' stamp: 'kph 10/21/2006 06:52'!
withAnswersDo: aBlock

	(aBlock respondsTo: #valueSuppressingMessages:supplyingAnswers: )
		ifTrue: [aBlock valueSuppressingMessages: self messagesToSuppress supplyingAnswers: self answers.]
		ifFalse: [ aBlock value ]
! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 10/21/2006 01:37'!
wsm
	"Answer the value of wsm"
	
	^ wsm! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 10/21/2006 01:37'!
wsm: aUrl
 
	"Set the value of host"

	wsm := aUrl last = $/ ifTrue: [ aUrl ] ifFalse: [ aUrl, '/' ]! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 12/18/2006 06:49'!
wsmDownloadUrl 

| pkgAndVersion packageId packageName packageVersion releaseAutoVersion
 downloadPage |

	pkgAndVersion := self smPackageAndVersion.
	packageName := pkgAndVersion first.
	packageVersion := pkgAndVersion last.
	packageVersion isEmpty ifTrue: [ packageVersion := #latest ].

	packageId := self wsmPackagesByName at: packageName.
	releaseAutoVersion := (self wsmReleasesFor: packageId) at: packageVersion.
					 
	downloadPage := self httpGet: (self wsm,'packagebyname/', packageName,'/autoversion/', releaseAutoVersion,'/downloadurl') asUrl asString.
				 		 
	^ downloadPage contents
	
! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 1/8/2007 06:43'!
wsmInstall 

	| downloadUrl |

	self logCR: 'finding ', self package, ' from websqueakmap(', self wsm, ') ...'.

	downloadUrl := self wsmDownloadUrl.
	
	self logCR: 'found at ', downloadUrl asString, ' ...'.
	 
	self install: downloadUrl from: (self httpGet: downloadUrl).

	! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 12/18/2006 06:51'!
wsmPackagesByName

	| html id name | 
	
	packages ifNotNil: [ ^packages ].
	
	packages := Dictionary new.
	
	html := self httpGet: (self wsm, 'packagesbyname').
	
	[
		id := html upToAll: '/package/'; upToAll: '">'.
		name := html upTo: $<.
		
		(id notEmpty and: [ name notEmpty ]) 

	] whileTrue: [ packages at: name put: id ].

	^ packages	
	! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 12/18/2006 06:50'!
wsmReleasesFor: packageId

	| html autoVersion version releases |
		
	releases := Dictionary new.
	
	html := self httpGet: (self wsm, '/package/', packageId ).
	
	[
		releases at: #latest put: autoVersion.
		autoVersion := html upToAll: '/autoversion/'; upTo: $".
		version := html upTo: $-; upTo: $<.
		
		(autoVersion notEmpty and: [version notEmpty ]) 

	] whileTrue: [ releases at: version put: autoVersion ].

	^ releases
	! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 10/21/2006 06:04'!
wsmVersions

| pkgAndVersion packageId packageName packageVersion versions |

	pkgAndVersion := self smPackageAndVersion.
	packageName := pkgAndVersion first.
	packageVersion := pkgAndVersion last.
	packageVersion isEmpty ifTrue: [ packageVersion := #latest ].

	packageId := self wsmPackagesByName at: packageName.
	
	versions := (self wsmReleasesFor: packageId) keys.
	versions remove: #latest.
	
	^ versions collect: [ :version | self copy package: (packageName,'(', version ,')'); yourself ]. ! !
Installer initialize!
